@@ -1,6 +1,6 @@
 /* Disk file handling
    Copyright (c) 1997-2004 Sandro Sigala.
-   Copyright (c) 2003-2006 Reuben Thomas.
+   Copyright (c) 2003-2007 Reuben Thomas.
    All rights reserved.
 
    This file is part of Zile.
@@ -20,7 +20,7 @@
    Software Foundation, Fifth Floor, 51 Franklin Street, Boston, MA
    02111-1301, USA.  */
 
-/*      $Id: file.c,v 1.81 2006/11/29 20:57:02 rrt Exp $        */
+/*      $Id: file.c,v 1.84 2007/01/19 15:32:50 rrt Exp $        */
 
 #include "config.h"
 
@@ -283,6 +283,20 @@ void open_file(char *path, size_t lineno)
   resync_redisplay();
 }
 
+#if HAVE_UNISTD_H
+/* Return nonzero if file exists and can be written. */
+static int check_writable(const char *filename)
+{
+#ifdef HAVE_EUIDACCESS
+  return euidaccess(filename, W_OK) >= 0;
+#else
+  /* Access isn't quite right because it uses the real uid
+     and we really want to test with the effective uid. */
+  return access(filename, W_OK) >= 0;
+#endif
+}
+#endif
+
 /*
  * Read the file contents into a buffer.
  * Return quietly if the file doesn't exist.
@@ -305,6 +319,11 @@ void read_from_disk(const char *filename)
     return;
   }
 
+#if HAVE_UNISTD_H
+  if (!check_writable(filename))
+    cur_bp->flags |= BFLAG_READONLY;
+#endif
+  
   lp = cur_bp->pt.p;
 
   while ((size = fread(buf, 1, BUFSIZ, fp)) > 0)
@@ -389,7 +408,8 @@ Completion *make_buffer_completion(void)
 
 DEFUN("find-file", find_file)
 /*+
-Edit a file specified by the user.  Switch to a buffer visiting the file,
+Edit the specified file.
+Switch to a buffer visiting the file,
 creating one if none already exists.
 +*/
 {
@@ -414,6 +434,20 @@ creating one if none already exists.
 }
 END_DEFUN
 
+DEFUN("find-file-read-only", find_file_read_only)
+/*+
+Edit the specified file but don't allow changes.
+Like `find-file' but marks buffer as read-only.
+Use M-x toggle-read-only to permit editing.
++*/
+{
+  int ret_value;
+  if ((ret_value = FUNCALL(find_file)))
+    cur_bp->flags |= BFLAG_READONLY;
+  return ret_value;
+}
+END_DEFUN
+
 DEFUN("find-alternate-file", find_alternate_file)
 /*+
 Find the file specified by the user, select its buffer, kill previous buffer.
@@ -901,7 +935,7 @@ static int write_to_disk(Buffer *bp, char *filename)
     astr_delete(bfilename);
   }
 
-  if (raw_write_to_disk(bp, filename, 0644) == FALSE) {
+  if (raw_write_to_disk(bp, filename, 0666) == FALSE) {
     minibuf_error("%s: %s", filename, strerror(errno));
     return FALSE;
   }
